# This file was auto-generated by Fern from our API Definition.

import typing

from .workflow_integration_dependency import WorkflowIntegrationDependency
from .workflow_model_provider_dependency import WorkflowModelProviderDependency

WorkflowDependency = typing.Union[WorkflowIntegrationDependency, WorkflowModelProviderDependency]
